@@ -5,14 +5,21 @@ import sys
 import subprocess
 import re
 
+from configparser import RawConfigParser
+from io import StringIO
+from urllib.parse import urlsplit
+
 from datetime import datetime
 from dateutil.tz import tzutc
 from dateutil.relativedelta import relativedelta
 from botocore.utils import parse_timestamp
 
-from awscli.compat import is_windows, urlparse, RawConfigParser, StringIO, get_stderr_encoding
+from awscli.compat import (
+    is_windows, urlparse, get_stderr_encoding, is_macos
+)
 from awscli.customizations import utils as cli_utils
 from awscli.customizations.commands import BasicCommand
+from awscli.utils import original_ld_library_path
 from awscli.customizations.utils import uni_print
 
 
@@ -45,7 +52,7 @@ class CommandFailedError(Exception):
         Exception.__init__(self, msg)
 
 
-class BaseLogin(object):
+class BaseLogin:
     _TOOL_NOT_FOUND_MESSAGE = '%s was not found. Please verify installation.'
 
     def __init__(self, auth_token, expiration, repository_endpoint,
@@ -95,11 +102,12 @@ class BaseLogin(object):
 
     def _run_command(self, tool, command, *, ignore_errors=False):
         try:
-            self.subprocess_utils.run(
-                command,
-                capture_output=True,
-                check=True
-            )
+            with original_ld_library_path():
+                self.subprocess_utils.run(
+                    command,
+                    capture_output=True,
+                    check=True,
+                )
         except subprocess.CalledProcessError as ex:
             if not ignore_errors:
                 raise CommandFailedError(ex)
@@ -115,6 +123,137 @@ class BaseLogin(object):
         raise NotImplementedError('get_commands()')
 
 
+class SwiftLogin(BaseLogin):
+
+    DEFAULT_NETRC_FMT = \
+        u'machine {hostname} login token password {auth_token}'
+
+    NETRC_REGEX_FMT = \
+        r'(?P<entry_start>\bmachine\s+{escaped_hostname}\s+login\s+\S+\s+password\s+)' \
+        r'(?P<token>\S+)'
+
+    def login(self, dry_run=False):
+        scope = self.get_scope(
+            self.namespace
+        )
+        commands = self.get_commands(
+            self.repository_endpoint, self.auth_token, scope=scope
+        )
+
+        if not is_macos:
+            hostname = urlparse.urlparse(self.repository_endpoint).hostname
+            new_entry = self.DEFAULT_NETRC_FMT.format(
+                hostname=hostname,
+                auth_token=self.auth_token
+            )
+            if dry_run:
+                self._display_new_netrc_entry(new_entry, self.get_netrc_path())
+            else:
+                self._update_netrc_entry(hostname, new_entry, self.get_netrc_path())
+
+        self._run_commands('swift', commands, dry_run)
+
+    def _display_new_netrc_entry(self, new_entry, netrc_path):
+        sys.stdout.write('Dryrun mode is enabled, not writing to netrc.')
+        sys.stdout.write(os.linesep)
+        sys.stdout.write(
+            f'The following line would have been written to {netrc_path}:'
+        )
+        sys.stdout.write(os.linesep)
+        sys.stdout.write(os.linesep)
+        sys.stdout.write(new_entry)
+        sys.stdout.write(os.linesep)
+        sys.stdout.write(os.linesep)
+        sys.stdout.write('And would have run the following commands:')
+        sys.stdout.write(os.linesep)
+        sys.stdout.write(os.linesep)
+
+    def _update_netrc_entry(self, hostname, new_entry, netrc_path):
+        pattern = re.compile(
+            self.NETRC_REGEX_FMT.format(escaped_hostname=re.escape(hostname)),
+            re.M
+        )
+        if not os.path.isfile(netrc_path):
+            self._create_netrc_file(netrc_path, new_entry)
+        else:
+            with open(netrc_path, 'r') as f:
+                contents = f.read()
+            escaped_auth_token = self.auth_token.replace('\\', r'\\')
+            new_contents = re.sub(
+                pattern,
+                rf"\g<entry_start>{escaped_auth_token}",
+                contents
+            )
+
+            if new_contents == contents:
+                new_contents = self._append_netrc_entry(new_contents, new_entry)
+
+            with open(netrc_path, 'w') as f:
+                f.write(new_contents)
+
+    def _create_netrc_file(self, netrc_path, new_entry):
+        dirname = os.path.split(netrc_path)[0]
+        if not os.path.isdir(dirname):
+            os.makedirs(dirname)
+        with os.fdopen(os.open(netrc_path,
+                               os.O_WRONLY | os.O_CREAT, 0o600), 'w') as f:
+            f.write(new_entry + '\n')
+
+    def _append_netrc_entry(self, contents, new_entry):
+        if contents.endswith('\n'):
+            return contents + new_entry + '\n'
+        else:
+            return contents + '\n' + new_entry + '\n'
+
+    @classmethod
+    def get_netrc_path(cls):
+        return os.path.join(os.path.expanduser("~"), ".netrc")
+
+    @classmethod
+    def get_scope(cls, namespace):
+        # Regex for valid scope name
+        valid_scope_name = re.compile(
+            r'\A[a-zA-Z0-9](?:[a-zA-Z0-9]|-(?=[a-zA-Z0-9])){0,38}\Z'
+        )
+
+        if namespace is None:
+            return namespace
+
+        if not valid_scope_name.match(namespace):
+            raise ValueError(
+                'Invalid scope name, scope must contain URL-safe '
+                'characters, no leading dots or underscores and no '
+                'more than 39 characters'
+            )
+
+        return namespace
+
+    @classmethod
+    def get_commands(cls, endpoint, auth_token, **kwargs):
+        commands = []
+        scope = kwargs.get('scope')
+
+        # Set up the codeartifact repository as the swift registry.
+        set_registry_command = [
+            'swift', 'package-registry', 'set', endpoint
+        ]
+        if scope is not None:
+            set_registry_command.extend(['--scope', scope])
+        commands.append(set_registry_command)
+
+        # Authenticate against the repository.
+        # We will write token to .netrc for Linux and Windows
+        # MacOS will store the token from command line option to Keychain
+        login_registry_command = [
+            'swift', 'package-registry', 'login', f'{endpoint}login'
+        ]
+        if is_macos:
+            login_registry_command.extend(['--token', auth_token])
+        commands.append(login_registry_command)
+
+        return commands
+
+
 class NuGetBaseLogin(BaseLogin):
     _NUGET_INDEX_URL_FMT = '{endpoint}v3/index.json'
 
@@ -164,11 +303,12 @@ class NuGetBaseLogin(BaseLogin):
             return
 
         try:
-            self.subprocess_utils.run(
-                command,
-                capture_output=True,
-                check=True
-            )
+            with original_ld_library_path():
+                self.subprocess_utils.run(
+                    command,
+                    capture_output=True,
+                    check=True
+                )
         except subprocess.CalledProcessError as e:
             uni_print('Failed to update the NuGet.Config\n')
             raise CommandFailedError(e)
@@ -197,10 +337,11 @@ class NuGetBaseLogin(BaseLogin):
         100. Source Name 100 [Activé]
              https://source100.com/index.json
         """
-        response = self.subprocess_utils.check_output(
-            self._get_list_command(),
-            stderr=self.subprocess_utils.PIPE
-        )
+        with original_ld_library_path():
+            response = self.subprocess_utils.check_output(
+                self._get_list_command(),
+                stderr=self.subprocess_utils.PIPE
+            )
 
         lines = response.decode(os.device_encoding(1) or "utf-8").splitlines()
         lines = [line for line in lines if line.strip() != '']
@@ -347,7 +488,7 @@ class NpmLogin(BaseLogin):
             [cls.NPM_CMD, 'config', 'set', registry, endpoint]
         )
 
-        repo_uri = urlparse.urlsplit(endpoint)
+        repo_uri = urlsplit(endpoint)
 
         # configure npm to always require auth for the repository.
         always_auth_config = '//{}{}:always-auth'.format(
@@ -380,7 +521,7 @@ class PipLogin(BaseLogin):
 
     @classmethod
     def get_commands(cls, endpoint, auth_token, **kwargs):
-        repo_uri = urlparse.urlsplit(endpoint)
+        repo_uri = urlsplit(endpoint)
         pip_index_url = cls.PIP_INDEX_URL_FMT.format(
             scheme=repo_uri.scheme,
             auth_token=auth_token,
@@ -393,7 +534,7 @@ class PipLogin(BaseLogin):
 
 class TwineLogin(BaseLogin):
 
-    DEFAULT_PYPI_RC_FMT = u'''\
+    DEFAULT_PYPI_RC_FMT = '''\
 [distutils]
 index-servers=
     pypi
@@ -417,7 +558,7 @@ password: {auth_token}'''
         if pypi_rc_path is None:
             pypi_rc_path = self.get_pypi_rc_path()
         self.pypi_rc_path = pypi_rc_path
-        super(TwineLogin, self).__init__(
+        super().__init__(
             auth_token, expiration, repository_endpoint,
             domain, repository, subprocess_utils)
 
@@ -464,7 +605,7 @@ password: {auth_token}'''
                 pypi_rc.set('codeartifact', 'username', 'aws')
                 pypi_rc.set('codeartifact', 'password', auth_token)
             except Exception as e:  # invalid .pypirc file
-                sys.stdout.write('%s is in an invalid state.' % pypi_rc_path)
+                sys.stdout.write(f'{pypi_rc_path} is in an invalid state.')
                 sys.stdout.write(os.linesep)
                 raise e
         else:
@@ -490,7 +631,7 @@ password: {auth_token}'''
             sys.stdout.write('Dryrun mode is enabled, not writing to pypirc.')
             sys.stdout.write(os.linesep)
             sys.stdout.write(
-                '%s would have been set to the following:' % self.pypi_rc_path
+                f'{self.pypi_rc_path} would have been set to the following:'
             )
             sys.stdout.write(os.linesep)
             sys.stdout.write(os.linesep)
@@ -511,6 +652,11 @@ class CodeArtifactLogin(BasicCommand):
     '''Log in to the idiomatic tool for the requested package format.'''
 
     TOOL_MAP = {
+        'swift': {
+            'package_format': 'swift',
+            'login_cls': SwiftLogin,
+            'namespace_support': True,
+        },
         'nuget': {
             'package_format': 'nuget',
             'login_cls': NuGetLogin,
